# __main__.py

import os
import sys
import pandas as pd
import generator
from importlib import resources  # Python 3.7+

def main():
    """Begin here"""

    # instantiate the class
    g = generator.Generator()

    # data directory
    data_dir = os.path.join(os.getcwd(), "data")
    
    # filename sentences without errors
    train_data = "nlpaeg_pubmed_data_min.csv"
    train_data_file_path = os.path.join(data_dir, train_data)

    # read as dataframe
    #df = pd.read_csv(train_data_file_path, sep="\t", header=None)
    #df.columns = ["source", "valid", "note", "sentence"]

    # set source data
    g.source_data = pd.read_csv(train_data_file_path)
    g.source_data = g.source_data.sample(1000)

    # set sentence column name
    # default: sentences
    g.sentence_column = "sentences"

    # define n-gram order
    # 4 => quadgrams, trigrams, bigrams and unigrams
    # 3 => trigrams, bigrams and unigrams
    # 2 => bigrams and unigrams
    # default is 3; max is 5
    g.ngram_order = 4

    # define proportion of ngram matches to modify
    # for example, if there were 10 sentences in total
    # changes to unigrams -> 10
    # changes to bigrams -> 7
    # changes to trigrams -> 3
    # we'll need to use all three trigrams, most of bigrams
    # and half of unigrams
    g.ngram_weights = {
        1: 0.4, 2: 0.6, 3: 0.8, 4: 0.95, 5: 1
    }


    # call the method to create error data
    aeg_df = g.get_aeg_data()

    print(aeg_df.head(2))

if __name__ == "__main__":
    main()


import random
import pandas as pd
import helper_variables as hv
from itertools import chain
from collections import Counter

# globals

# split words
splitter_keys = list(hv.splitters.keys())

# replacements
replacement_keys = list(hv.replacements.keys())

# operations
operations = hv.operations





# create n-grams
def create_ngrams(input_list, n):
    return list(zip(*[input_list[i:] for i in range(n)]))
    

# return sentences

def create_sent_ngrams(ngram_order, ngram_cols, sentence_column, source_data, n_ngrams):
    # dictionary of most common n n-grams
    ngrams = {}

    df = pd.DataFrame()
    df[sentence_column] = source_data[sentence_column]

    # iterate over ngram order to create df columns
    # with names from the dict ngram_cols
    for n in range(1, ngram_order+1):
        df[ngram_cols[n]] = source_data[sentence_column].map(lambda x: create_ngrams(x.split(" "), n))
        grams = df[ngram_cols[n]].tolist()
        grams = list(chain(*grams))
        grams = [x for x in grams]
        grams_counter = Counter(grams)
        ngrams[ngram_cols[n]] = grams_counter.most_common(n_ngrams[n])
        ngrams[ngram_cols[n]] = [ng for ng, count in ngrams[ngram_cols[n]]]

    # combine sentences with ngrams
    # select random most common ngram for each sentence
    ncols = df.columns
    sent_ngrams = list(df[ncols].values)

    return ngrams, sent_ngrams


# sent list of sentences and its ngrams
# return randomly chosen most common ngram in the sentence

def random_most_common_ngram(sent_ngrams, ngram_cols, ngram_order, ngrams):
    
    # required output
    # [ngram matched, sentence, replacement words]
    replacements = []

    # sent_ngrams = [sentence, [1-gm], [2-gms], [3-gms], [4-gms], [5-gms]]
    # return replacements = [ngram matched, sentence, replacement words]
    # found replacements
    found = []

    for indx, sngs in enumerate(sent_ngrams):

        for i in range(ngram_order, 0, -1):
            ngs = sngs[i]
            ncom = list(set(ngrams[ngram_cols[i]]).intersection(ngs))

            #if indx not in found:
            if ncom:
                req_one_ncom = random.choice(ncom)
                replacements.append([indx, i, sngs[0], req_one_ncom])
                found.append(indx)                    
        #else:
            #    continue


    return replacements

#sent_ngrams = list(df[["sentences", "quadgrams", "trigrams", "bigrams", "unigrams"]].values)
#random_most_common_ngram(sent_ngrams[35:43])







# variables


adj_tags = ['JJ', 'JJR', 'JJS']
noun_tags = ['NN', 'NNP', 'NNS', 'NNPS']
verb_tags = ['VB', 'VBD', 'VBG', 'VBN', 'VBP', 'VBZ']
adverb_tags = ['RB', 'RBR', 'RBS']

# operations
operations = ["dict_replacement", "duplication",
              "punctuations", "punctuation_braces",  
              "random_removal", "split_words",
              "insert_determiner", "change_order",

             ]



# split words
splitters = {
    "healthcare": "health care",
    "trademark": "trade mark",
    "decision-making": "decision making",
    "health-related": "health related",
    "highlight": "high light",
    "endpoint": "end point",
    "biomarker": "bio marker",
    "without": "with out",
    "meta-analysis": "meta analysis",
    "non-alcoholic": "non alcoholic",
    "understand": "under stand",
    "understood": "under stood",
    "relationship": "relation ship",
    
}


# were -> where
# there -> they are

replacements = {
    
    "have": ["has"],
    "has": ["have"],
    "were": ["we are", "we're", "where",],
    "we're": ["were",],
    "where": ["were", "we're", "we are", ],
    "there": ["their", "they're", "they are"],
    "their": ["there", "they're", "they are"],
    "a": ["an", "as", "are", ],
    "an": ["a", "are", ],
    "its": ["it's", "it is"],
    "it's": ["its"],
    "to": ["too", "on",],
    "too": ["to",],
    "of": ["off",],
    "off": ["of",],
    "effect": ["affect",],
    "affect": ["effect",],
    "by": ["buy", "bye"],
    "buy": ["by", "bye"],
    "here": ["hear", ],
    "hear": ["here", ],
    
    
}







# generator.py

# load libraries
import numpy as np
import pandas as pd
import helper_functions as hf


'''
var= np.round(np.random.normal(distribution mean, 
standard deviation, number of samples), 
number of digits after the decimal point)


# .5 is prob dist of 0 and 1
label = np.random.choice([0, 1], size=7000, p=[.5, .5]) 


'''

class Generator():
    def __init__(self):
        self.name = "nlpaeg"
        self.source_data = pd.DataFrame()
        self.aeg_data = pd.DataFrame()

        self.total_samples = len(self.source_data)

        # how many ngrams to consider
        # using most frequent ones
        # total unigrams -> 1000; take top 300
        self.n_ngrams = {
            1: int(self.total_samples * 0.3),
            2: int(self.total_samples * 0.2),
            3: int(self.total_samples * 0.15),
            4: int(self.total_samples * 0.1),
            5: int(self.total_samples * 0.05),
        }

        # for sampling ngram changes
        self.ngram_weights = {
            1: 0.4, # 40% of unigram changes
            2: 0.6, # 60% of bigram changes
            3: 0.8, # 80% of trigram changes
            4: 0.95, # 95% of quadgram changes
            5: 1    # 100% of pentgram changes
        }

        self.sentence_column = "sentences"


        # ngram_order: 1 to 5
        self.ngram_order = 3
        self.ngram_cols = {
            1: "unigrams", 2: "bigrams", 3: "trigrams", 
            4: "quadgrams", 5: "pentgrams"
        }

        self.error_distribution = {

        }


    

    def get_aeg_data(self):
        '''

        '''        


        self.total_samples = len(self.source_data)
        self.n_ngrams = {
            1: int(self.total_samples * 0.3),
            2: int(self.total_samples * 0.2),
            3: int(self.total_samples * 0.15),
            4: int(self.total_samples * 0.1),
            5: int(self.total_samples * 0.05),
        }

        # ngram order <= 5; >=1
        if not (self.ngram_order > 0 and self.ngram_order < 6):
            print("n-gram order should be between 0 and 6")
            raise

        
        ngrams, sent_ngrams = hf.create_sent_ngrams(self.ngram_order, self.ngram_cols,
                                            self.sentence_column, self.source_data,
                                            self.n_ngrams,
                                            )
    
        replacements = hf.random_most_common_ngram(sent_ngrams[:100], self.ngram_cols, 
                                                    self.ngram_order, ngrams)

        tmp_df = pd.DataFrame(replacements, columns=["index", "ngram", "sentence", "replace"])

        sampled_replacements = pd.DataFrame()
        tmp = dict()

        for i in range(1, self.ngram_order+1):
            tmp[i] = tmp_df.loc[tmp_df.ngram == i].sample(frac=self.ngram_weights[i])
            sampled_replacements = pd.concat([sampled_replacements, tmp[i]], ignore_index=True)


        #self.aeg_data = self.source_data.sample()

        return sampled_replacements
